import os
import json
from typing import Optional

def load_config() -> dict:
    config_path = os.path.expanduser("~/.super_md_config.json")
    
    if os.path.exists(config_path):
        with open(config_path, 'r') as f:
            return json.load(f)
    return {}

def save_config(config: dict):
    config_path = os.path.expanduser("~/.super_md_config.json")
    
    with open(config_path, 'w') as f:
        json.dump(config, f, indent=2)



def remove_config():
    config_path = os.path.expanduser("~/.super_md_config.json")
    
    with open(config_path, 'w') as f:
        f.write("")
    
    os.remove(config_path)

def get_auth_token() -> Optional[str]:
    config = load_config()
    return config.get("token")

def save_auth_token(token: str):
    config = load_config()
    config["token"] = token
    save_config(config)


import requests
import base64
import json
from typing import Optional, Dict, List
import os
from dotenv import load_dotenv

load_dotenv()
BASE_URL = "http://localhost:8080"

class APIClient:
    def __init__(self, base_url: Optional[str] = None):
        self.base_url = BASE_URL
        self.token = None
        self.session = requests.Session()
    
    def login(self, username: str, password: str) -> bool:
        try:
            response = self.session.post(
                f"{self.base_url}/oauth/authenticate/",
                json={"username": username, "password": password}
            )
            response.raise_for_status()
            
            data = response.json()
            print(data)
            
            self.token = data.get("data").get("access_token")
            self.refresh_token = data.get("data").get("refresh_token")
            print(self.token)
            
            if self.token:
                self.session.headers.update({
                    "Authorization": f"Bearer {self.token}",
                    "Content-Type": "application/json"
                })
                return True
            return False
            
        except requests.exceptions.RequestException as e:
            print(f"Erreur de connexion: {e}")
            return False


    def publish_document(self, file_path: str, folder: str = "default") -> bool:
        try:
            # Vérifier si le fichier existe
            if not os.path.exists(file_path):
                print(f"Error: The file {file_path} does not exist")
                return False
            
            # Vérifier l'extension
            if not file_path.endswith('.md') and not file_path.endswith('.smd'):
                print("Error: The file must have the extension .md / .smd")
                return False
            
            # Lire et encoder le fichier
            with open(file_path, 'r', encoding='utf-8') as f:
                content = f.read()
            
            encoded_content = base64.b64encode(content.encode('utf-8')).decode('utf-8')
            filename = os.path.basename(file_path)
            
            # Envoyer au serveur
            payload = {
                "filename": filename,
                "data": encoded_content,
            }
            print(payload)
            
            response = self.session.post(
                f"{self.base_url}/api/space/upload-publication/",
                json=payload
            )
            response.raise_for_status()
            
            print(f"Document {filename} successfully published")
            return True
            
        except requests.exceptions.RequestException as e:
            print(f"Error during the publish process: {e}")
            return False
        except Exception as e:
            print(f"Unexpected error: {e}")
            return False
    
    def list_documents(self, folder: Optional[str] = "space") -> List[Dict]:
        model = "space"
        try:
            params = {}
            if folder:
                params['space'] = folder
            

            
            if folder != None:
                model = "publication"
            

            payload = {
                "query" : "",
                "model" : model,
                "type" : "",
                "parent" : folder,
            }

            if model == "publication" : 
                response = self.session.post(
                    f"{self.base_url}/api/space/get-space/",
                    params=params,
                    json=payload
                )
                response.raise_for_status()
                data = response.json().get("data")
            else:
                if payload["parent"] == None:
                    payload["parent"] = "-"
                response = self.session.post(
                    f"{self.base_url}/api/space/search-model/",
                    params=params,
                    json=payload
                )
                response.raise_for_status()
                data = response.json().get("data").get("owned")
            
            return data
            
        except requests.exceptions.RequestException as e:
            print(f"Error retrieving documents: {e}")
            return []


import argparse
import getpass
import sys
from typing import Optional
from rich.console import Console
from rich.table import Table

from super_md.api_client import APIClient
from super_md.utils import save_auth_token
from super_md.utils import get_auth_token
from super_md.utils import remove_config


BASE_URL = "http://localhost:8080"

console = Console()

def login_command(args):
    username = args.username or input("username: ")
    password = args.password or getpass.getpass("Password: ")
    
    client = APIClient()
    
    if client.login(username, password):
        if client.token:
            save_auth_token(client.token)
        console.print("[green]✓ Connection successful![/green]")
    else:
        console.print("[red]✗ Login failed[/red]")
        sys.exit(1)

def publish_command(args):
    client = APIClient()
    
    # Charger le token depuis la configuration
    token = get_auth_token()

    if token == None:
        console.print("[red]Your are not connected, please login to proceed this operation[/red]")
        return
    
    if token:
        client.token = token
        client.session.headers.update({
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        })
    
    if client.publish_document(args.file, args.folder):
        console.print("[green]✓ Publication complete[/green]")
    else:
        console.print("[red]✗ Publication failed[/red]")
        sys.exit(1)
def logout_command(args):

    remove_config()

    console.print("[yellow]Your account has been logged out[/yellow]")

def show_command(args):
    client = APIClient()
    token = get_auth_token()
    
    if token == None:
        console.print("[red]Your are not connected, please login to proceed this operation[/red]")
        return
    
    if token:
        client.token = token
        client.session.headers.update({
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json"
        })
    
    documents = client.list_documents(args.folder)

    target_model = "space"

    if args.folder != None:
        target_model = "publication"
    
    if documents:
        
        listing = []
        if target_model == "space":
            listing = documents["space"]
            
            table = Table(title=f"Folders listing :", show_lines=True)
            table.add_column("Nom", style="white")
            table.add_column("Description", style="yellow")
            table.add_column("Goal", style="green")
            table.add_column("Link", style="white")
            
            for doc in listing:
                table.add_row(
                    doc.get('name', 'N/A'),
                    doc.get('description', 'N/A'),
                    doc.get('goal', 'N/A'),
                    BASE_URL+doc.get('link', 'N/A'),
                )
        
        if target_model == "publication":
            listing = documents["publication"][0]
            listing_space = documents["space"][0][0]
            
            listing_folders = documents["sub_space"][0]
            print(listing_folders)
            table = Table(title=f"Details Folder "+listing_space.get('name', 'N/A')+" :", show_lines=True)
            table.add_column("Name / Title", style="white")
            table.add_column("Folder", style="yellow")
            table.add_column("Link", style="white")

            for subs in listing_folders:
                table.add_row(
                    "[bold yellow]"+subs.get('name', 'N/A')+"[/bold yellow] (Folder)",
                    listing_space.get('name', 'N/A'),
                    "[bold green]"+BASE_URL+subs.get('link', 'N/A')+"[/bold green]",
                )
            
            for doc in listing:
                table.add_row(
                    doc.get('title', 'N/A'),
                    listing_space.get('name', 'N/A'),
                    BASE_URL+doc.get('link', 'N/A'),
                )
        
        console.print(table)
    else:
        console.print("[yellow]No publications found[/yellow]")

def main():
    #print("Super MD CLI")
    parser = argparse.ArgumentParser(
        description="CLI for interacting with the Super Markdown platform API"
    )
    subparsers = parser.add_subparsers(dest="command", help="Commands available")
    
    # Commande login
    login_parser = subparsers.add_parser("login", help="Authentication")
    login_parser.add_argument("-u", "--username", help="Username")
    login_parser.add_argument("-p", "--password", help="Password")
    login_parser.set_defaults(func=login_command)
    
    # Commande publish
    publish_parser = subparsers.add_parser("publish", help="Publish a Markdown document")
    publish_parser.add_argument("file", help="Path to the *.md / *.smd file to publish")
    publish_parser.add_argument("-f", "--folder", 
                               default="default",
                               help="Destination folder (default: default)")
    publish_parser.set_defaults(func=publish_command)
    
    # Commande show
    show_parser = subparsers.add_parser("show", help="List of publications")
    show_parser.add_argument("-f", "--folder",  help="Target folder name (optional)")
    show_parser.set_defaults(func=show_command)


    # Commande logout
    logout_parser = subparsers.add_parser("logout", help="Disconnect from your account locally")
    logout_parser.set_defaults(func=logout_command)
    
    # Parser les arguments
    args = parser.parse_args()
    
    if hasattr(args, 'func'):
        args.func(args)
    else:
        parser.print_help()

if __name__ == "__main__":
    main()